From 7f6872599b9ec87557a010c800e2d74cb5985d5f Mon Sep 17 00:00:00 2001
From: Alexander Polakov <polachok@gmail.com>
Date: Sun, 12 Feb 2012 14:18:30 +0400
Subject: [PATCH 02/10] Focus MRU client when doing group_only()

---
 calmwm.h |    1 +
 client.c |   26 ++++++++++++++++++++++++++
 group.c  |    6 ++++++
 3 files changed, 33 insertions(+), 0 deletions(-)

diff --git a/calmwm.h b/calmwm.h
index ea32a40..dee49db 100644
--- a/calmwm.h
+++ b/calmwm.h
@@ -319,6 +319,7 @@ void			 client_map(struct client_ctx *);
 void			 client_maximize(struct client_ctx *);
 void			 client_move(struct client_ctx *);
 void			 client_mtf(struct client_ctx *);
+struct client_ctx 	*client_mrusearch(struct screen_ctx *, struct group_ctx *);
 struct client_ctx	*client_new(Window, struct screen_ctx *, int);
 void			 client_ptrsave(struct client_ctx *);
 void			 client_ptrwarp(struct client_ctx *);
diff --git a/client.c b/client.c
index 531432d..0de46dc 100644
--- a/client.c
+++ b/client.c
@@ -646,6 +646,32 @@ client_cycle(struct screen_ctx *sc, int flags)
 	client_ptrwarp(newcc);
 }
 
+struct client_ctx *
+client_mrusearch(struct screen_ctx *sc, struct group_ctx *gc) 
+{
+	int again = 1;
+	struct client_ctx *cc;
+
+	if (TAILQ_EMPTY(&sc->mruq))
+		return NULL;
+
+	cc = TAILQ_FIRST(&sc->mruq);
+	while (again) {
+		again = 0;
+
+		/* Only cycle visible and non-ignored windows. */
+		if ((cc->flags & (CLIENT_HIDDEN|CLIENT_IGNORE))
+				&& cc->group != gc)
+			again = 1;
+		else
+			break;
+		cc = client_mrunext(cc);
+		if (cc == TAILQ_FIRST(&sc->mruq))
+			return NULL;
+	}
+	return cc;
+}
+
 static struct client_ctx *
 client_mrunext(struct client_ctx *cc)
 {
diff --git a/group.c b/group.c
index 0d606d0..3cbf7ac 100644
--- a/group.c
+++ b/group.c
@@ -309,16 +309,22 @@ void
 group_only(struct screen_ctx *sc, int idx)
 {
 	int	 i;
+	struct group_ctx 	*gc;
+	struct client_ctx 	*cc;
 
 	if (idx < 0 || idx >= CALMWM_NGROUPS)
 		err(1, "group_only: index out of range (%d)", idx);
 
+	gc = &sc->groups[idx];
+
 	for (i = 0; i < CALMWM_NGROUPS; i++) {
 		if (i == idx)
 			group_show(sc, &sc->groups[i]);
 		else
 			group_hide(sc, &sc->groups[i]);
 	}
+	if ((cc = client_mrusearch(sc, gc)) != NULL)
+		client_ptrwarp(cc);
 }
 
 /*
-- 
1.7.6

